@@ -7,7 +7,7 @@
     and (certain) typed namedtuples.
     The JSON-encoding preserves all information necessary to reconstruct the at decoding time (cf. `typing_json.decoding.from_json_obj`).
 
-    (Version: 0.1.0)
+    (Version: 0.1.1)
 """
 
 # standard imports
@@ -116,7 +116,34 @@ def is_json_encodable(t: Type, failure_callback: Optional[Callable[[str], None]]
             return True
     return False
 
-def to_json_obj(obj: Any, t: Type, use_decimal: bool = False) -> Any:
+
+def _to_json_obj_namedtuple(obj, field_types, use_decimal=True, namedtuples_as_lists=False):
+    # pylint:disable=invalid-name
+    if namedtuples_as_lists:
+        return [to_json_obj(getattr(obj, field), field_type, use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for field, field_type in field_types.items()]
+    json_dict = OrderedDict() # type:ignore
+    for field, field_type in field_types.items():
+        json_dict[field] = to_json_obj(getattr(obj, field), field_type, use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists)
+    return json_dict
+
+
+def _to_json_obj_homogeneous_collection(obj, element_t, use_decimal=True, namedtuples_as_lists=False):
+    # pylint:disable=invalid-name,too-many-return-statements
+    if element_t in JSON_BASE_TYPES or element_t in (None, type(None)):
+        return list(obj)
+    if element_t is Decimal:
+        if use_decimal:
+            return list(obj)
+        return [str(el) for el in obj]
+    if isinstance(element_t, EnumMeta):
+        return [el._name_ for el in obj] # pylint:disable=protected-access
+    if is_namedtuple(element_t):
+        field_types = getattr(element_t, "_field_types")
+        return [_to_json_obj_namedtuple(el, field_types, use_decimal=use_decimal, namedtuples_as_lists=namedtuples_as_lists) for el in obj]
+    return [to_json_obj(x, element_t, use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for x in obj]
+
+
+def to_json_obj(obj: Any, t: Type, use_decimal: bool = False, typecheck: bool = True, namedtuples_as_lists=False) -> Any:
     """
         Encodes an instance `obj` of typecheckable type `t` into a JSON object.
         The optional `use_decimal` parameter can be used to specify that instances of
@@ -124,9 +151,6 @@ def to_json_obj(obj: Any, t: Type, use_decimal: bool = False) -> Any:
         This method raises `TypeError` if type `t` is not typecheckable according to `typing_json.typechecking.is_typecheckable`.
         This method raises `TypeError` if `obj` is not of type `t` according to `typing_json.typechecking.is_instance`.
 
-        The optional parameter `failure_callback` can be used to collect a detailed trace of
-        the reasons behind this method returning `False` on a given type `t`.
-
         Currently, this method acts as follows on an instance `obj` of type `t`:
 
         - if `t` is one of the JSON basic types `bool`, `int`, `float`, `str`, `NoneType`, the instance `obj` is returned unchanged;
@@ -134,7 +158,8 @@ def to_json_obj(obj: Any, t: Type, use_decimal: bool = False) -> Any:
         - if `t` is `decimal.Decimal` and `use_decimal` is `True`, `obj` is returned unchanged;
         - if `t` is `None` (used as an alias for `NoneType`), `None` is returned;
         - if `t` is an enum (i.e. `isinstance(t, EnumMeta)`), the enum value name `obj._name_` is returned;
-        - if `t` is a namedtuple according to `typing_json.typechecking.is_namedtuple` and all its fields are JSON encodable, this method is called recursively on all field values and then an ordered dictionary is returned with the field names as names and the JSON-encoded field values as corresponding values;
+        - if `t` is a namedtuple according to `typing_json.typechecking.is_namedtuple` and all its fields are JSON encodable and `namedtuples_as_lists` is `False`, this method is called recursively on all field values and then an ordered dictionary is returned with the field names as names and the JSON-encoded field values as corresponding values;
+        - if `t` is a namedtuple according to `typing_json.typechecking.is_namedtuple` and all its fields are JSON encodable and `namedtuples_as_lists` is `True`, this method is called recursively on all field values and then a list is returned with the JSON-encoded field values appearing in the same order as the namedtuple fields (which are not explicitly encoded);
         - if `t` is `typing.Union`, the generic type arguments in the union are tried one after the other until a `u` is found such that `is_instance(obj, u)`, then `obj` is JSON-encoded using `u` as its type.
         - if `t` is a `typing_extensions.Literal`, `obj` is returned unchanged;
         - if `t` is one of `typing.List`, `typing.Set`, `typing.FrozenSet`, `typing.Deque` or `typing.Tuple`, a list is returned containing the elements of the original collection, recursively JSON-encoded;
@@ -149,18 +174,22 @@ def to_json_obj(obj: Any, t: Type, use_decimal: bool = False) -> Any:
 
         Literals can only be of JSON basic type.
 
+        An optional parameter `typecheck` (default: `True`) can be used to skip the check that `t` be JSON encodable and that `obj` be an instance of `t`.
+        The parameter `typecheck` is set to `False` in all recursive calls (i.e. typechecking is only done once).
+
     """
     # pylint:disable=invalid-name,too-many-return-statements,too-many-branches
-    trace: List[str] = []
-    def failure_callback(message: str) -> None:
-        trace.append(message)
-    if not is_json_encodable(t, failure_callback=failure_callback):
-        # Argument `t` must be JSON encodable.
-        raise TypeError("Type %s is not json-encodable. Trace:\n%s"%(str(t), "\n".join(trace)))
-    trace = []
-    if not is_instance(obj, t, failure_callback=failure_callback):
-        # Argument `obj` must be an instance of argument `t`.
-        raise TypeError("Object %s is not of type %s. Trace:\n%s"%(short_str(obj), str(t), "\n".join(trace)))
+    if typecheck:
+        trace: List[str] = []
+        def failure_callback(message: str) -> None:
+            trace.append(message)
+        if not is_json_encodable(t, failure_callback=failure_callback):
+            # Argument `t` must be JSON encodable.
+            raise TypeError("Type %s is not json-encodable. Trace:\n%s"%(str(t), "\n".join(trace)))
+        trace = []
+        if not is_instance(obj, t, failure_callback=failure_callback):
+            # Argument `obj` must be an instance of argument `t`.
+            raise TypeError("Object %s is not of type %s. Trace:\n%s"%(short_str(obj), str(t), "\n".join(trace)))
     if t in JSON_BASE_TYPES:
         # JSON basic types are returned unchanged.
         return obj
@@ -179,30 +208,27 @@ def to_json_obj(obj: Any, t: Type, use_decimal: bool = False) -> Any:
     if is_namedtuple(t):
         # Namedtuples are encoded as ordered dictionaries, with their fields as keys and the JSON-encoded field values as corresponding values.
         field_types = getattr(t, "_field_types")
-        json_dict = OrderedDict() # type:ignore
-        for field in field_types:
-            json_dict[field] = to_json_obj(getattr(obj, field), field_types[field], use_decimal=use_decimal)
-        return json_dict
+        return _to_json_obj_namedtuple(obj, field_types, use_decimal=use_decimal, namedtuples_as_lists=namedtuples_as_lists)
     if hasattr(t, "__origin__") and hasattr(t, "__args__"):
         # Generics from the `typing` module.
         if t.__origin__ is Union:
             # values in a `typing.Union` are JSON-encoded using the first type in the union that the object is found to be an instance of.
             for s in t.__args__:
                 if is_instance(obj, s):
-                    return to_json_obj(obj, s, use_decimal=use_decimal)
+                    return to_json_obj(obj, s, use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists)
             raise AssertionError(_UNREACHABLE_ERROR_MSG) # pragma: no cover
         if t.__origin__ is Literal:
             # `typing_extensions.Literal` are returned unchanged
             return obj
         if t.__origin__ in (list, set, frozenset, deque):
             # `typing.List`, `typing.Set`, `typing.FrozenSet` and `typing.Deque` are turned into lists, with their elements recursively JSON-encoded
-            return [to_json_obj(x, t.__args__[0], use_decimal=use_decimal) for x in obj]
+            return _to_json_obj_homogeneous_collection(obj, t.__args__[0], use_decimal=use_decimal, namedtuples_as_lists=namedtuples_as_lists)
         if t.__origin__ is tuple:
             # `typing.Tuple` are turned into lists, with their elements recursively JSON-encoded
             if len(t.__args__) == 2 and t.__args__[1] is ...: # pylint:disable=no-else-return
-                return [to_json_obj(x, t.__args__[0], use_decimal=use_decimal) for x in obj]
+                return _to_json_obj_homogeneous_collection(obj, t.__args__[0], use_decimal=use_decimal, namedtuples_as_lists=namedtuples_as_lists)
             else:
-                return [to_json_obj(x, t.__args__[i], use_decimal=use_decimal) for i, x in enumerate(obj)]
+                return [to_json_obj(x, t.__args__[i], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for i, x in enumerate(obj)]
         if t.__origin__ in (dict, OrderedDict, Mapping):
             # `typing.Dict` and `typing.Mapping` are turned into dictionaries and `typing.OrderedDict` are turned into ordered dictionaries.
             # The values are recursively JSON-encoded. Keys require special handling.
@@ -210,24 +236,24 @@ def to_json_obj(obj: Any, t: Type, use_decimal: bool = False) -> Any:
             if t.__args__[0] in JSON_BASE_TYPES+(Decimal, None,):
                 # Keys of JSON basic types, `decimal.Decimal` and `None` are recursively JSON-encoded.
                 # encoded_fields = [field for field in fields] # pylint: disable = unnecessary-comprehension
-                encoded_fields = [to_json_obj(field, t.__args__[0], use_decimal=use_decimal) for field in fields]
+                encoded_fields = [to_json_obj(field, t.__args__[0], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for field in fields]
             elif (hasattr(t.__args__[0], "__origin__") and t.__args__[0].__origin__ is Literal):
                 # Keys of `typing_extensions.Literal` types are recursively JSON-encoded.
                 # encoded_fields = [field for field in fields] # pylint: disable = unnecessary-comprehension
-                encoded_fields = [to_json_obj(field, t.__args__[0], use_decimal=use_decimal) for field in fields]
+                encoded_fields = [to_json_obj(field, t.__args__[0], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for field in fields]
             elif isinstance(t.__args__[0], EnumMeta):
                 # Keys of enumeration types are recursively JSON-encoded.
-                encoded_fields = [to_json_obj(field, t.__args__[0], use_decimal=use_decimal) for field in fields]
+                encoded_fields = [to_json_obj(field, t.__args__[0], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for field in fields]
             else:
                 # Keys of any other type are recursively JSON-encoded and then JSON dumped to strings.
-                encoded_fields = [json.dumps(to_json_obj(field, t.__args__[0], use_decimal=use_decimal)) for field in fields]
+                encoded_fields = [json.dumps(to_json_obj(field, t.__args__[0], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists)) for field in fields]
             if t.__origin__ in (dict, Mapping):
                 # A `dict`is used for `typing.Dict` and `typing.Mapping`.
-                return {encoded_fields[i]: to_json_obj(obj[field], t.__args__[1], use_decimal=use_decimal) for i, field in enumerate(fields)}
+                return {encoded_fields[i]: to_json_obj(obj[field], t.__args__[1], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists) for i, field in enumerate(fields)}
             if t.__origin__ is OrderedDict:
                 # A `collections.OrderedDict` is used for `typing.OrderedDict`.
                 new_ordered_dict = OrderedDict() # type:ignore
                 for i, field in enumerate(fields):
-                    new_ordered_dict[encoded_fields[i]] = to_json_obj(obj[field], t.__args__[1], use_decimal=use_decimal)
+                    new_ordered_dict[encoded_fields[i]] = to_json_obj(obj[field], t.__args__[1], use_decimal=use_decimal, typecheck=False, namedtuples_as_lists=namedtuples_as_lists)
                 return new_ordered_dict
     raise AssertionError(_UNREACHABLE_ERROR_MSG) # pragma: no cover
